"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HAP_1 = require("./HAP");
const Logger_1 = require("./Logger");
const Client_1 = require("./Client");
const Mutex_1 = require("./util/Mutex");
class LockAccessory {
    constructor(lock, accessory) {
        this.lock = lock;
        this.accessory = accessory;
        this.client = new Client_1.Client();
        this.mutex = new Mutex_1.Mutex();
        this.accessory.getService(HAP_1.HAP.Service.AccessoryInformation)
            .setCharacteristic(HAP_1.HAP.Characteristic.Manufacturer, 'CANDY HOUSE')
            .setCharacteristic(HAP_1.HAP.Characteristic.Model, 'Sesame')
            .setCharacteristic(HAP_1.HAP.Characteristic.SerialNumber, this.lock.serial);
        this.setupLockServiceCharacteristics();
        this.setupBatteryServiceCharacteristics();
    }
    getOrCreateHAPService(service) {
        let hapService = this.accessory.getService(service);
        if (!hapService) {
            hapService = this.accessory.addService(service, this.lock.nickname);
        }
        return hapService;
    }
    setupLockServiceCharacteristics() {
        let lockService = this.getOrCreateHAPService(HAP_1.HAP.Service.LockMechanism);
        lockService.getCharacteristic(HAP_1.HAP.Characteristic.LockCurrentState)
            .on('get', this.getCurrentLockState.bind(this));
        lockService.getCharacteristic(HAP_1.HAP.Characteristic.LockTargetState)
            .on('get', this.getTargetLockState.bind(this))
            .on('set', this.setLockState.bind(this));
    }
    setupBatteryServiceCharacteristics() {
        let batteryService = this.getOrCreateHAPService(HAP_1.HAP.Service.BatteryService);
        batteryService.getCharacteristic(HAP_1.HAP.Characteristic.BatteryLevel)
            .on('get', this.getBatteryLevel.bind(this));
        batteryService.getCharacteristic(HAP_1.HAP.Characteristic.ChargingState)
            .on('get', this.getBatteryChargingState.bind(this));
        batteryService.getCharacteristic(HAP_1.HAP.Characteristic.StatusLowBattery)
            .on('get', this.getLowBatteryStatus.bind(this));
    }
    getCurrentLockState(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let status;
            try {
                status = yield this.mutex.wait(() => this.client.getStatus(this.lock.device_id));
            }
            catch (e) {
                Logger_1.Logger.error('Unable to get lock state', e);
                callback(e);
            }
            if (!status.responsive) {
                Logger_1.Logger.log(`${this.lock.nickname} is unresponsive, forcing a status sync...`);
                try {
                    let result = yield this.client.sync(this.lock.device_id);
                    if (result.successful) {
                        Logger_1.Logger.log(`${this.lock.nickname} sync successful.`);
                    }
                    else {
                        Logger_1.Logger.error(`${this.lock.nickname} failed to sync, please check WiFi connectivity. API responded with: ${result.error}`);
                    }
                }
                catch (e) {
                    Logger_1.Logger.error(`Unable to sync`, e);
                }
            }
            if (status.locked) {
                callback(null, HAP_1.HAP.Characteristic.LockCurrentState.SECURED);
            }
            else {
                callback(null, HAP_1.HAP.Characteristic.LockCurrentState.UNSECURED);
            }
        });
    }
    getTargetLockState(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.mutex.wait(() => this.client.getStatus(this.lock.device_id));
                if (status.locked) {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.SECURED);
                }
                else {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.UNSECURED);
                }
            }
            catch (e) {
                Logger_1.Logger.error('Unable to get lock state', e);
                callback(e);
            }
        });
    }
    setLockState(targetState, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.Logger.log(`${this.lock.nickname} is ${targetState ? 'locking' : 'unlocking'}...`);
            try {
                yield this.client.control(this.lock.device_id, targetState);
            }
            catch (e) {
                Logger_1.Logger.error('Unable to set lock state', e);
                callback(e);
            }
            Logger_1.Logger.log(`${this.lock.nickname} is ${targetState ? 'locked' : 'unlocked'}`);
            this.updateCurrentLockState(targetState);
            callback();
        });
    }
    updateCurrentLockState(locked) {
        let lockService = this.getOrCreateHAPService(HAP_1.HAP.Service.LockMechanism);
        lockService.getCharacteristic(HAP_1.HAP.Characteristic.LockCurrentState)
            .updateValue(locked);
    }
    getBatteryLevel(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.mutex.wait(() => this.client.getStatus(this.lock.device_id));
                callback(null, status.battery);
            }
            catch (e) {
                callback(e);
            }
        });
    }
    getLowBatteryStatus(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.mutex.wait(() => this.client.getStatus(this.lock.device_id));
                if (status.battery <= 20) {
                    callback(null, HAP_1.HAP.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW);
                }
                else {
                    callback(null, HAP_1.HAP.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
                }
            }
            catch (e) {
                callback(e);
            }
        });
    }
    getBatteryChargingState(callback) {
        callback(null, HAP_1.HAP.Characteristic.ChargingState.NOT_CHARGING);
    }
}
exports.LockAccessory = LockAccessory;
//# sourceMappingURL=LockAccessory.js.map